"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Just_1 = require("../../src/algebra/Just");
const RealMaybe_1 = require("../../src/algebra/RealMaybe");
const RealNothing_1 = require("../../src/algebra/RealNothing");
const composeUsingReverse_1 = require("../../src/functions/composeUsingReverse");
const curry_1 = require("../../src/functions/curry");
const identity_1 = require("../../src/functions/identity");
describe('Maybe<T> is a Functor because: ', () => {
    it(' Maybe.map respects the Functor identity law for Functor', () => {
        const maybeOf1 = RealMaybe_1.Maybe.of(1);
        expect(maybeOf1.map(identity_1.identity).chain(identity_1.identity)).toBe(maybeOf1.chain(identity_1.identity));
    });
    it(' Maybe<T>.map respects the Functor identity law for Nothing', () => {
        const maybeOfNull = RealMaybe_1.Maybe.of(null);
        expect(maybeOfNull.map(identity_1.identity)).toEqual(maybeOfNull);
    });
    it(' Maybe<T>.map respects the Functor composition law for Functor', () => {
        const add1 = x => x + 1;
        const mult2 = x => x * 2;
        const add1ThenMult2 = composeUsingReverse_1.composeUsingReverse(mult2, add1);
        const maybeOf1 = RealMaybe_1.Maybe.of(1);
        const idMaybe1 = maybeOf1.map(add1ThenMult2);
        expect(maybeOf1.map(add1ThenMult2)).toEqual(maybeOf1
            .map(add1)
            .map(mult2));
        expect(maybeOf1.map(add1ThenMult2).value).toBe(4);
    });
    it(' Maybe<T>.map respects the Functor composition law for Nothing', () => {
        const add1 = x => x + 1;
        const mult2 = x => x * 2;
        const add1ThenMult2 = composeUsingReverse_1.composeUsingReverse(mult2, add1);
        const maybeOfNull = RealMaybe_1.Maybe.of(null);
        expect(maybeOfNull.map(add1ThenMult2)).toEqual(maybeOfNull
            .map(add1)
            .map(mult2));
        expect(maybeOfNull.map(add1ThenMult2).value).toBe(null);
    });
    it(' Maybe.map can safely decide if Nothing should be returned or Just', () => {
        const someObj = {
            something: {
                somethingElse: {}
            }
        };
        const safeProp = curry_1.curry(function safeProp(prop, obj) {
            return RealMaybe_1.Maybe.of(obj[prop]);
        });
        const res = RealMaybe_1.Maybe.of(someObj)
            .chain(safeProp('something'))
            .chain(safeProp('somethingElse'));
        expect(res instanceof Just_1.Just).toBe(true);
        expect(res.chain(safeProp('more')) instanceof RealNothing_1.Nothing).toBe(true);
    });
});
//# sourceMappingURL=RealMaybe.spec.js.map