"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Applicative_1 = require("../../src/examples/Applicative");
const identity_1 = require("../../src/functions/identity");
const compose2_1 = require("../../src/functions/compose2");
const curry_1 = require("../../src/functions/curry");
describe('Applicative: ', () => {
    it(' Respects the Identity Law', () => {
        const applicativeOfId = Applicative_1.Applicative.of(identity_1.identity);
        const functorOf4 = Applicative_1.Applicative.of(4);
        expect(applicativeOfId.ap(functorOf4)).toEqual(functorOf4);
    });
    it(' Respects the Homomorphism Law', () => {
        const f = x => x + 2;
        const applicativeOfF = Applicative_1.Applicative.of(f);
        const applicativeOf1 = Applicative_1.Applicative.of(1);
        expect(applicativeOfF.ap(applicativeOf1)).toEqual(Applicative_1.Applicative.of(f(1)));
    });
    it(' Respects the Interchange Law', () => {
        const f = x => x + 2;
        const applicativeOfF = Applicative_1.Applicative.of(f);
        const applicativeOf1 = Applicative_1.Applicative.of(1);
        expect(applicativeOfF.ap(Applicative_1.Applicative.of(1))).toEqual(Applicative_1.Applicative.of(f).ap(applicativeOf1));
    });
    it(' Respects the Composition Law', () => {
        const compose = curry_1.curry(compose2_1.compose2);
        const u = Applicative_1.Applicative.of((x) => x.toUpperCase());
        const v = Applicative_1.Applicative.of((x) => x.concat('& beyond'));
        const w = Applicative_1.Applicative.of('blood bath');
        expect(Applicative_1.Applicative.of(compose)
            .ap(u)
            .ap(v)
            .ap(w)).toEqual(u.ap(v.ap(w)));
    });
});
//# sourceMappingURL=Applicative.spec.js.map